@@ -51,22 +51,26 @@
  */
 
 #include <gtk/gtkstyle.h>
+#include <gal/util/e-i18n.h>
+#include <gal/util/e-util.h>
 #include "e-paned.h"
 
 enum {
-  ARG_0,
-  ARG_HANDLE_SIZE,
-  ARG_QUANTUM,
+	PROP_0,
+	PROP_HANDLE_SIZE,
+	PROP_QUANTUM
 };
 
 static void    e_paned_class_init (EPanedClass  *klass);
 static void    e_paned_init       (EPaned       *paned);
-static void    e_paned_set_arg    (GtkObject      *object,
-				     GtkArg         *arg,
-				     guint           arg_id);
-static void    e_paned_get_arg    (GtkObject      *object,
-				     GtkArg         *arg,
-				     guint           arg_id);
+static void    e_paned_set_property (GObject *object,
+				     guint prop_id,
+				     const GValue *value,
+				     GParamSpec *pspec);
+static void    e_paned_get_property (GObject *object,
+				     guint prop_id,
+				     GValue *value,
+				     GParamSpec *pspec);
 static void    e_paned_realize    (GtkWidget      *widget);
 static void    e_paned_map        (GtkWidget      *widget);
 static void    e_paned_unmap      (GtkWidget      *widget);
@@ -86,55 +90,43 @@ static void    e_paned_style_set (GtkWidget *widget,
 
 static GtkType e_paned_child_type (GtkContainer   *container);
 
+#define PARENT_TYPE GTK_TYPE_CONTAINER
 static GtkContainerClass *parent_class = NULL;
 
 
-GtkType
-e_paned_get_type (void)
-{
-  static GtkType paned_type = 0;
-  
-  if (!paned_type)
-    {
-      static const GtkTypeInfo paned_info =
-      {
-	"EPaned",
-	sizeof (EPaned),
-	sizeof (EPanedClass),
-	(GtkClassInitFunc) e_paned_class_init,
-	(GtkObjectInitFunc) e_paned_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
-      };
-
-      paned_type = gtk_type_unique (GTK_TYPE_CONTAINER, &paned_info);
-    }
-  
-  return paned_type;
-}
+E_MAKE_TYPE (e_paned,
+	     "EPaned",
+	     EPaned,
+	     e_paned_class_init,
+	     e_paned_init,
+	     PARENT_TYPE)
 
 static void
 e_paned_class_init (EPanedClass *klass)
 {
-  GtkObjectClass *object_class;
+  GObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
 
-  object_class = (GtkObjectClass *) klass;
+  object_class = (GObjectClass *) klass;
   widget_class = (GtkWidgetClass *) klass;
   container_class = (GtkContainerClass *) klass;
 
-  parent_class = gtk_type_class (GTK_TYPE_CONTAINER);
+  parent_class = g_type_class_ref (PARENT_TYPE);
 
-  object_class->set_arg = e_paned_set_arg;
-  object_class->get_arg = e_paned_get_arg;
+  object_class->set_property = e_paned_set_property;
+  object_class->get_property = e_paned_get_property;
 
   widget_class->realize = e_paned_realize;
   widget_class->map = e_paned_map;
   widget_class->unmap = e_paned_unmap;
   widget_class->unrealize = e_paned_unrealize;
+#ifndef NO_WARNINGS
+#warning removed expose
+#endif
+#if 0
   widget_class->expose_event = e_paned_expose;
+#endif
   widget_class->style_set = e_paned_style_set;
 
   container_class->add = e_paned_add;
@@ -144,10 +136,19 @@ e_paned_class_init (EPanedClass *klass)
 
   klass->handle_shown = NULL;
 
-  gtk_object_add_arg_type("EPaned::handle_size", GTK_TYPE_UINT,
-			  GTK_ARG_READWRITE, ARG_HANDLE_SIZE);
-  gtk_object_add_arg_type("EPaned::quantum", GTK_TYPE_UINT,
-			  GTK_ARG_READWRITE, ARG_QUANTUM);
+  g_object_class_install_property (object_class, PROP_HANDLE_SIZE, 
+				   g_param_spec_uint ("handle_size",
+						      _("Handle Size"),
+						      /*_( */"XXX blurb" /*)*/,
+						      0, G_MAXUINT, 0,
+						      G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class, PROP_QUANTUM,
+				   g_param_spec_uint ("quantum",
+						      _("Quantum"),
+						      /*_( */"XXX blurb" /*)*/,
+						      0, G_MAXUINT, 0,
+						      G_PARAM_READWRITE));
 }
 
 static GtkType
@@ -186,46 +187,46 @@ e_paned_init (EPaned *paned)
 }
 
 static void
-e_paned_set_arg (GtkObject *object,
-		   GtkArg    *arg,
-		   guint      arg_id)
+e_paned_set_property (GObject *object,
+		      guint prop_id,
+		      const GValue *value,
+		      GParamSpec *pspec)
 {
-  EPaned *paned = E_PANED (object);
+	EPaned *paned = E_PANED (object);
   
-  switch (arg_id)
-    {
-    case ARG_HANDLE_SIZE:
-      e_paned_set_handle_size (paned, GTK_VALUE_UINT (*arg));
-      break;
-    case ARG_QUANTUM:
-      paned->quantum = GTK_VALUE_UINT (*arg);
-      if (paned->quantum == 0)
-	paned->quantum = 1;
-      break;
-    default:
-      break;
-    }
+	switch (prop_id) {
+	case PROP_HANDLE_SIZE:
+		e_paned_set_handle_size (paned, g_value_get_uint (value));
+		break;
+	case PROP_QUANTUM:
+		paned->quantum = g_value_get_uint (value);
+		if (paned->quantum == 0)
+			paned->quantum = 1;
+		break;
+	default:
+		break;
+	}
 }
 
 static void
-e_paned_get_arg (GtkObject *object,
-		   GtkArg    *arg,
-		   guint      arg_id)
+e_paned_get_property (GObject *object,
+		      guint prop_id,
+		      GValue *value,
+		      GParamSpec *pspec)
 {
-  EPaned *paned = E_PANED (object);
-
-  switch (arg_id)
-    {
-    case ARG_HANDLE_SIZE:
-      GTK_VALUE_UINT (*arg) = paned->handle_size;
-      break;
-    case ARG_QUANTUM:
-      GTK_VALUE_UINT (*arg) = paned->quantum;
-      break;
-    default:
-      arg->type = GTK_TYPE_INVALID;
-      break;
-    }
+	EPaned *paned = E_PANED (object);
+	
+	switch (prop_id) {
+	case PROP_HANDLE_SIZE:
+		g_value_set_uint (value, paned->handle_size);
+		break;
+	case PROP_QUANTUM:
+		g_value_set_uint (value, paned->quantum);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 static void
@@ -362,9 +363,13 @@ e_paned_unrealize (GtkWidget *widget)
     (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
 }
 
+#ifndef NO_WARNINGS
+#warning FIXME: badly broken
+#endif
+#if 0
 static gint
 e_paned_expose (GtkWidget      *widget,
-		  GdkEventExpose *event)
+		GdkEventExpose *event)
 {
   EPaned *paned;
   GdkEventExpose child_event;
@@ -404,6 +409,7 @@ e_paned_expose (GtkWidget      *widget,
 
   return FALSE;
 }
+#endif
 
 void
 e_paned_add1 (EPaned  *paned,
@@ -654,7 +660,7 @@ e_paned_compute_position(EPaned *paned,
 gboolean
 e_paned_handle_shown(EPaned *paned)
 {
-  EPanedClass *klass = E_PANED_CLASS(GTK_OBJECT(paned)->klass);
+  EPanedClass *klass = E_PANED_CLASS(GTK_OBJECT_GET_CLASS(paned));
 
   if (klass->handle_shown)
     return (*klass->handle_shown)(paned);
