@@ -47,7 +47,6 @@
 #include "e-cell-text.h"
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-canvas.h"
-#include "gal/widgets/e-font.h"
 #include "gal/widgets/e-unicode.h"
 #include "e-table-item.h"
 #include "gal/util/e-text-event-processor.h"
@@ -80,6 +79,7 @@ enum {
 	PROP_0,
 
 	PROP_STRIKEOUT_COLUMN,
+	PROP_UNDERLINE_COLUMN,
 	PROP_BOLD_COLUMN,
 	PROP_COLOR_COLUMN,
 	PROP_EDITABLE,
@@ -111,7 +111,6 @@ typedef struct _CellEdit CellEdit;
 typedef struct {
 	ECellView    cell_view;
 	GdkGC       *gc;
-	EFont *font;
 	GdkCursor *i_cursor;
 	GdkBitmap *stipple;		/* Stipple for text */
 	
@@ -192,7 +191,6 @@ static void _get_tep (CellEdit *edit);
 static gint get_position_from_xy (CellEdit *edit, gint x, gint y);
 static gboolean _blink_scroll_timeout (gpointer data);
 
-static void calc_ellipsis (ECellTextView *text_view);
 static void ect_free_color (gchar *color_spec, GdkColor *color, GdkColormap *colormap);
 static GdkColor* e_cell_text_get_color (ECellTextView *cell_view, gchar *color_spec);
 
@@ -375,22 +373,11 @@ static void
 ect_realize (ECellView *ecell_view)
 {
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
-	ECellText *ect = (ECellText *) ecell_view->ecell;
 	
 	text_view->gc = gdk_gc_new (GTK_WIDGET (text_view->canvas)->window);
 
 	text_view->i_cursor = gdk_cursor_new (GDK_XTERM);
 	
-	if (ect->font_name){
-		text_view->font = e_font_from_gdk_name (ect->font_name);
-	}
-	if (!text_view->font){
-		gdk_font_ref (gtk_style_get_font (GTK_WIDGET (text_view->canvas)->style));
-		text_view->font = e_font_from_gdk_font (gtk_style_get_font (GTK_WIDGET (text_view->canvas)->style));
-	}
-	
-	calc_ellipsis (text_view);
-
 	if (parent_class->realize)
 		(* parent_class->realize) (ecell_view);
 }
@@ -412,10 +399,6 @@ ect_unrealize (ECellView *ecv)
 		ect_cancel_edit (text_view);
 	}
 
-	if (text_view->font)
-		e_font_unref (text_view->font);
-	text_view->font = NULL;
-	
 	if (text_view->stipple)
 		gdk_bitmap_unref (text_view->stipple);
 
@@ -456,7 +439,7 @@ build_layout (ECellTextView *text_view, int row, const char *text)
 	ECellView *ecell_view = (ECellView *) text_view;
 	ECellText *ect = E_CELL_TEXT (ecell_view->ecell);
 	PangoLayout *layout;
-	gboolean bold, strikeout;
+	gboolean bold, strikeout, underline;
 
 	layout = gtk_widget_create_pango_layout (GTK_WIDGET (((GnomeCanvasItem *)ecell_view->e_table_item_view)->canvas), text);
 
@@ -466,8 +449,11 @@ build_layout (ECellTextView *text_view, int row, const char *text)
 	strikeout = ect->strikeout_column >= 0 &&
 		row >= 0 &&
 		e_table_model_value_at(ecell_view->e_table_model, ect->strikeout_column, row);
+	underline = ect->underline_column >= 0 &&
+		row >= 0 &&
+		e_table_model_value_at(ecell_view->e_table_model, ect->underline_column, row);
 
-	if (bold || strikeout) {
+	if (bold || strikeout || underline) {
 		PangoAttrList *attrs;
 		int length = strlen (text);
 		attrs = pango_attr_list_new ();
@@ -485,6 +471,13 @@ build_layout (ECellTextView *text_view, int row, const char *text)
 
 			pango_attr_list_insert_before (attrs, attr);
 		}
+		if (underline) {
+			PangoAttribute *attr = pango_attr_underline_new (TRUE);
+			attr->start_index = 0;
+			attr->end_index = length;
+
+			pango_attr_list_insert_before (attrs, attr);
+		}
 		pango_layout_set_attributes (layout, attrs);
 		pango_attr_list_unref (attrs);
 	}
@@ -1283,9 +1276,9 @@ ect_show_tooltip (ECellView *ecell_view,
 	tooltip_text = gnome_canvas_item_new (gnome_canvas_root (GNOME_CANVAS (canvas)),
 					      e_text_get_type (),
 					      "anchor", GTK_ANCHOR_NW,
-/*  					      "font_gdk", text_view->font, */
 					      "bold", (gboolean) ect->bold_column >= 0 && e_table_model_value_at(ecell_view->e_table_model, ect->bold_column, row),
 					      "strikeout", (gboolean) ect->strikeout_column >= 0 && e_table_model_value_at(ecell_view->e_table_model, ect->strikeout_column, row),
+					      "underline", (gboolean) ect->underline_column >= 0 && e_table_model_value_at(ecell_view->e_table_model, ect->underline_column, row),
 					      "fill_color_gdk", tooltip->foreground,
 					      "text", pango_layout_get_text (layout),
 					      "editable", FALSE,
@@ -1363,6 +1356,10 @@ ect_set_property (GObject *object,
 		text->strikeout_column = g_value_get_int (value);
 		break;
 
+	case PROP_UNDERLINE_COLUMN:
+		text->underline_column = g_value_get_int (value);
+		break;
+
 	case PROP_BOLD_COLUMN:
 		text->bold_column = g_value_get_int (value);
 		break;
@@ -1400,6 +1397,10 @@ ect_get_property (GObject *object,
 		g_value_set_int (value, text->strikeout_column);
 		break;
 
+	case PROP_UNDERLINE_COLUMN:
+		g_value_set_int (value, text->underline_column);
+		break;
+
 	case PROP_BOLD_COLUMN:
 		g_value_set_int (value, text->bold_column);
 		break;
@@ -1469,6 +1470,13 @@ e_cell_text_class_init (GObjectClass *object_class)
 							   0, G_MAXINT, 0,
 							   G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class, PROP_UNDERLINE_COLUMN,
+					 g_param_spec_int ("underline_column",
+							   _("Underline Column"),
+							   /*_( */"XXX blurb" /*)*/,
+							   0, G_MAXINT, 0,
+							   G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class, PROP_BOLD_COLUMN,
 					 g_param_spec_int ("bold_column",
 							   _("Bold Column"),
@@ -1516,6 +1524,7 @@ e_cell_text_init (ECellText *ect)
 	ect->ellipsis = g_strdup (ellipsis_default);
 	ect->use_ellipsis = use_ellipsis_default;
 	ect->strikeout_column = -1;
+	ect->underline_column = -1;
 	ect->bold_column = -1;
 	ect->color_column = -1;
 	ect->bg_color_column = -1;
@@ -1527,7 +1536,7 @@ E_MAKE_TYPE(e_cell_text, "ECellText", ECellText, e_cell_text_class_init, e_cell_
 /**
  * e_cell_text_construct:
  * @cell: The cell to construct
- * @fontname: font to be used to render on the screen
+ * @fontname: this param is no longer used, but left here for api stability
  * @justify: Justification of the string in the cell
  *
  * constructs the ECellText.  To be used by subclasses and language
@@ -1545,7 +1554,7 @@ e_cell_text_construct (ECellText *cell, const char *fontname, GtkJustification j
 
 /**
  * e_cell_text_new:
- * @fontname: font to be used to render on the screen
+ * @fontname: this param is no longer used, but left here for api stability
  * @justify: Justification of the string in the cell.
  *
  * Creates a new ECell renderer that can be used to render strings that
@@ -1555,15 +1564,16 @@ e_cell_text_construct (ECellText *cell, const char *fontname, GtkJustification j
  * The ECellText object support a large set of properties that can be
  * configured through the Gtk argument system and allows the user to have
  * a finer control of the way the string is displayed.  The arguments supported
- * allow the control of strikeout, bold, and color.
+ * allow the control of strikeout, underline, bold, and color.
  *
- * The arguments "strikeout_column", "bold_column" and "color_column" set
- * and return an integer that points to a column in the model that controls
- * these settings.  So controlling the way things are rendered is achieved
- * by having special columns in the model that will be used to flag whether
- * the text should be rendered with strikeout, or bolded.   In the case of
- * the "color_column" argument, the column in the model is expected to have
- * a string that can be parsed by gdk_color_parse().
+ * The arguments "strikeout_column", "underline_column", "bold_column"
+ * and "color_column" set and return an integer that points to a
+ * column in the model that controls these settings.  So controlling
+ * the way things are rendered is achieved by having special columns
+ * in the model that will be used to flag whether the text should be
+ * rendered with strikeout, or bolded.  In the case of the
+ * "color_column" argument, the column in the model is expected to
+ * have a string that can be parsed by gdk_color_parse().
  * 
  * Returns: an ECell object that can be used to render strings.
  */
@@ -1701,14 +1711,11 @@ _get_position (ECellTextView *text_view, ETextEventProcessorCommand *command)
 {
 	int length;
 	CellEdit *edit = text_view->edit;
-	EFont *font;
 	gchar *p;
 	int unival;
 	int index;
 	int trailing;
 	
-	font = text_view->font;
-	
 	switch (command->position) {
 		
 	case E_TEP_VALUE:
@@ -1953,10 +1960,7 @@ e_cell_text_view_command (ETextEventProcessor *tep, ETextEventProcessorCommand *
 	gboolean redraw = FALSE;
 
 	int sel_start, sel_end;
-	EFont *font;
 	
-	font = text_view->font;
-
 	/* If the EText isn't editable, then ignore any commands that would
 	   modify the text. */
 	if (!ect->editable && (command->action == E_TEP_DELETE
@@ -2240,25 +2244,6 @@ _get_tep (CellEdit *edit)
 	}
 }
 
-static void
-calc_ellipsis (ECellTextView *text_view)
-{
-	ECellText *ect = E_CELL_TEXT (((ECellView *)text_view)->ecell);
-	EFont *font;
-	
-	font = text_view->font;
-	if (font) {
-		text_view->ellipsis_width[E_FONT_PLAIN] =
-			e_font_utf8_text_width (font, E_FONT_PLAIN,
-					ect->ellipsis ? ect->ellipsis : "...",
-					ect->ellipsis ? strlen (ect->ellipsis) : 3);
-		text_view->ellipsis_width[E_FONT_BOLD] =
-			e_font_utf8_text_width (font, E_FONT_BOLD,
-					ect->ellipsis ? ect->ellipsis : "...",
-					ect->ellipsis ? strlen (ect->ellipsis) : 3);
-	}
-}
-
 static GdkColor*
 e_cell_text_get_color (ECellTextView *cell_view, gchar *color_spec)
 {
