@@ -40,7 +40,7 @@
 #include <gtk/gtkinvisible.h>
 #include <gtk/gtksignal.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnomeui/gnome-canvas.h>
+#include <libgnomecanvas/gnome-canvas.h>
 
 #include "e-tree-table-adapter.h"
 #include "e-tree-model.h"
@@ -94,6 +94,9 @@ visible_depth_of_node (ETableModel *model, int row)
 		- (e_tree_table_adapter_root_node_is_visible (adapter) ? 0 : 1));
 }
 
+/* If this is changed to not include the width of the expansion pixmap
+   if the path is not expandable, then max_width needs to change as
+   well. */
 static gint
 offset_of_node (ETableModel *table_model, int row)
 {
@@ -195,7 +198,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	ETreeModel *tree_model = e_cell_tree_get_tree_model(ecell_view->e_table_model, row);
 	ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
 	ETreePath node;
-	GdkRectangle rect, *clip_rect;
+	GdkRectangle rect, *clip_rect = NULL;
 	GtkWidget *canvas = GTK_WIDGET (tree_view->canvas);
 	GdkGC *fg_gc = canvas->style->fg_gc[GTK_STATE_ACTIVE];
 	GdkColor *foreground;
@@ -330,6 +333,11 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	e_cell_draw (tree_view->subcell_view, drawable,
 		     model_col, view_col, row, flags,
 		     x1 + subcell_offset, y1, x2, y2);
+
+	if (clip_rect) {
+		gdk_gc_set_clip_rectangle (tree_view->gc, NULL);
+		gdk_gc_set_clip_rectangle (fg_gc, NULL);
+	}
 }
 
 /*
@@ -425,27 +433,29 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 	int number_of_rows;
 	int max_width = 0;
 	int width = 0;
-	int subcell_max_width;
+	int subcell_max_width = 0;
+	gboolean per_row = e_cell_max_width_by_row_implemented (tree_view->subcell_view);
 
 	number_of_rows = e_table_model_row_count (ecell_view->e_table_model);
 
-	subcell_max_width = e_cell_max_width (tree_view->subcell_view, model_col, view_col);
+	if (!per_row)
+		subcell_max_width = e_cell_max_width (tree_view->subcell_view, model_col, view_col);
 	
 	for (row = 0; row < number_of_rows; row++) {
 		ETreeModel *tree_model = e_cell_tree_get_tree_model(ecell_view->e_table_model, row);
-		ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
 		ETreePath node;
 		GdkPixbuf *node_image;
 		int node_image_width = 0, node_image_height = 0;
 		
 		int offset, subcell_offset;
+#if 0
 		gboolean expanded, expandable;
+		ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
+#endif
 		
 		node = e_cell_tree_get_node (ecell_view->e_table_model, row);
 		
 		offset = offset_of_node (ecell_view->e_table_model, row);
-		expandable = e_tree_model_node_is_expandable (tree_model, node);
-		expanded = e_tree_table_adapter_node_is_expanded (tree_table_adapter, node);
 		subcell_offset = offset;
 
 		node_image = e_tree_model_icon_at (tree_model, node);
@@ -455,7 +465,20 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 			node_image_height = gdk_pixbuf_get_height (node_image);
 		}
 
-		width = subcell_max_width + subcell_offset + node_image_width;
+		width = subcell_offset + node_image_width;
+
+		if (per_row)
+			width += e_cell_max_width_by_row (tree_view->subcell_view, model_col, view_col, row);
+		else
+			width += subcell_max_width;
+
+#if 0
+		expandable = e_tree_model_node_is_expandable (tree_model, node);
+		expanded = e_tree_table_adapter_node_is_expanded (tree_table_adapter, node);
+
+		/* This is unnecessary since this is already handled
+                   by the offset_of_node function.  If that changes,
+                   this will have to change too. */
 
 		if (expandable) {
 			GdkPixbuf *image;
@@ -466,6 +489,7 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 
 			width += gdk_pixbuf_get_width(image);
 		}
+#endif
 
 		max_width = MAX (max_width, width);
 	}
@@ -636,30 +660,35 @@ ect_print_height (ECellView *ecell_view, GnomePrintContext *context,
 }
 
 /*
- * GtkObject::destroy method
+ * GObject::dispose method
  */
 static void
-ect_destroy (GtkObject *object)
+ect_dispose (GObject *object)
 {
 	ECellTree *ect = E_CELL_TREE (object);
 
 	/* destroy our subcell */
 	if (ect->subcell)
-		gtk_object_unref (GTK_OBJECT (ect->subcell));
+		g_object_unref (ect->subcell);
 	ect->subcell = NULL;
 
-	gdk_pixbuf_unref (ect->open_pixbuf);
-	gdk_pixbuf_unref (ect->closed_pixbuf);
+	if (ect->open_pixbuf)
+		gdk_pixbuf_unref (ect->open_pixbuf);
+	ect->open_pixbuf = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	if (ect->closed_pixbuf)
+		gdk_pixbuf_unref (ect->closed_pixbuf);
+	ect->closed_pixbuf = NULL;
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
-e_cell_tree_class_init (GtkObjectClass *object_class)
+e_cell_tree_class_init (GObjectClass *object_class)
 {
 	ECellClass *ecc = (ECellClass *) object_class;
 
-	object_class->destroy = ect_destroy;
+	object_class->dispose = ect_dispose;
 
 	ecc->new_view         = ect_new_view;
 	ecc->kill_view        = ect_kill_view;
@@ -676,10 +705,10 @@ e_cell_tree_class_init (GtkObjectClass *object_class)
 	ecc->show_tooltip     = ect_show_tooltip;
 	ecc->get_bg_color     = ect_get_bg_color;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 }
 
-E_MAKE_TYPE(e_cell_tree, "ECellTree", ECellTree, e_cell_tree_class_init, NULL, PARENT_TYPE);
+E_MAKE_TYPE(e_cell_tree, "ECellTree", ECellTree, e_cell_tree_class_init, NULL, PARENT_TYPE)
 
 /**
  * e_cell_tree_construct:
@@ -702,7 +731,7 @@ e_cell_tree_construct (ECellTree *ect,
 {		       
 	ect->subcell = subcell;
 	if (subcell) {
-		gtk_object_ref (GTK_OBJECT (subcell));
+		g_object_ref (subcell);
 		gtk_object_sink (GTK_OBJECT (subcell));
 	}
 	if (open_pixbuf)
@@ -740,7 +769,7 @@ e_cell_tree_new (GdkPixbuf *open_pixbuf,
 		 gboolean draw_lines,
 		 ECell *subcell)
 {
-	ECellTree *ect = gtk_type_new (e_cell_tree_get_type ());
+	ECellTree *ect = g_object_new (E_CELL_TREE_TYPE, NULL);
 
 	e_cell_tree_construct (ect, open_pixbuf, closed_pixbuf, draw_lines, subcell);
 
