@@ -156,7 +156,7 @@ config_class_init (GObjectClass *object_class)
 							      E_TABLE_STATE_TYPE,
 							      G_PARAM_READABLE));
 
-	glade_gnome_init ();
+	glade_init ();
 }
 
 static ETableColumnSpecification *
@@ -517,7 +517,7 @@ create_global_store (ETableConfig *config)
 
 	global_store = e_table_memory_store_new (store_columns);
 	for (i = 0; config->source_spec->columns[i]; i++) {
-		char *text = e_utf8_from_locale_string (dgettext (config->domain,
+		char *text = g_strdup (dgettext (config->domain,
 								  config->source_spec->columns[i]->title));
 
 		e_table_memory_store_insert_adopt (E_TABLE_MEMORY_STORE (global_store), i, NULL, text);
