@@ -236,6 +236,8 @@ kill_gnode(GNode *node, ETreeTableAdapter *etta)
 	}
 
 	g_free(node->data);
+	if (node == etta->priv->root)
+		etta->priv->root = NULL;
 	g_node_destroy(node);
 }
 
@@ -442,15 +444,48 @@ insert_node(ETreeTableAdapter *etta, ETreePath parent, ETreePath path)
 	e_table_model_rows_inserted(E_TABLE_MODEL(etta), get_row(etta, path), size);
 }
 
+typedef struct {
+	GSList *paths;
+	gboolean expanded;
+} check_expanded_closure;
+
+static gboolean
+check_expanded(GNode *gnode, gpointer data)
+{
+	check_expanded_closure *closure = (check_expanded_closure *) data;
+	node_t *node = (node_t *) gnode->data;
+
+	if (node->expanded != closure->expanded)
+		closure->paths = g_slist_prepend(closure->paths, node->path);
+
+	return FALSE;
+}
+
 static void
 update_node(ETreeTableAdapter *etta, ETreePath path)
 {
+	check_expanded_closure closure;
 	ETreePath parent = e_tree_model_node_get_parent(etta->priv->source, path);
-	delete_node(etta, parent, path);
+	GNode *gnode = lookup_gnode(etta, path);
+	GSList *l;
+
+	closure.expanded = e_tree_model_get_expanded_default (etta->priv->source);
+	closure.paths = NULL;
+
+	if (gnode)
+		g_node_traverse(gnode, G_IN_ORDER, G_TRAVERSE_ALL, -1, check_expanded, &closure);
+
 	if (e_tree_model_node_is_root(etta->priv->source, path))
 		generate_tree(etta, path);
-	else
+	else {
+		delete_node(etta, parent, path);
 		insert_node(etta, parent, path);
+	}
+
+	for (l = closure.paths; l; l = l->next)
+		e_tree_table_adapter_node_set_expanded (etta, l->data, !closure.expanded);
+
+	g_slist_free(closure.paths);
 }
 
 static void
@@ -764,6 +799,9 @@ etta_proxy_node_request_collapse (ETreeModel *etm, ETreePath node, ETreeTableAda
 static void
 etta_sort_info_changed (ETableSortInfo *sort_info, ETreeTableAdapter *etta)
 {
+	if (!etta->priv->root)
+		return;
+
 	e_table_model_pre_change(E_TABLE_MODEL(etta));
 	resort_node(etta, etta->priv->root, TRUE);
 	fill_map(etta, 0, etta->priv->root);
@@ -1082,10 +1120,11 @@ e_tree_table_adapter_show_node (ETreeTableAdapter *etta, ETreePath path)
 gboolean     
 e_tree_table_adapter_node_is_expanded (ETreeTableAdapter *etta, ETreePath path)
 {
-	if (!e_tree_model_node_is_expandable (etta->priv->source, path))
+	node_t *node = get_node(etta, path);
+	if (!e_tree_model_node_is_expandable (etta->priv->source, path) || !node)
 		return FALSE;
 
-	return get_node(etta, path)->expanded;
+	return node->expanded;
 }
 
 void
