@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * gal-view-etable.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -22,40 +22,67 @@
  */
 
 #include <config.h>
-#include <gtk/gtksignal.h>
 #include "gal-view-etable.h"
 #include <gal/e-table/e-table-config.h>
 
-#define PARENT_TYPE gal_view_get_type ()
+#define PARENT_TYPE GAL_VIEW_TYPE
 
 static GalViewClass *gal_view_etable_parent_class;
 
+static void
+detach_table (GalViewEtable *view)
+{
+	if (view->table == NULL)
+		return;
+	if (view->table_state_changed_id) {
+		g_signal_handler_disconnect (view->table,
+					     view->table_state_changed_id);
+		view->table_state_changed_id = 0;
+	}
+	g_object_unref (view->table);
+	view->table = NULL;
+}
+
+static void
+detach_tree (GalViewEtable *view)
+{
+	if (view->tree == NULL)
+		return;
+	if (view->tree_state_changed_id) {
+		g_signal_handler_disconnect (view->tree,
+					     view->tree_state_changed_id);
+		view->tree_state_changed_id = 0;
+	}
+	g_object_unref (view->tree);
+	view->tree = NULL;
+}
 
 static void
 config_changed (ETableConfig *config, GalViewEtable *view)
 {
 	ETableState *state;
 	if (view->state)
-		gtk_object_unref(GTK_OBJECT(view->state));
-	gtk_object_get (GTK_OBJECT (config),
-			"state", &state,
-			NULL);
+		g_object_unref(view->state);
+	g_object_get (config,
+		      "state", &state,
+		      NULL);
 	view->state = e_table_state_duplicate(state);
 	gal_view_changed(GAL_VIEW(view));
 }
 
 static void
-gal_view_etable_edit            (GalView *view)
+gal_view_etable_edit            (GalView *view, GtkWindow *parent)
 {
 	GalViewEtable *etable_view = GAL_VIEW_ETABLE(view);
 	ETableConfig *config;
 
 	config = e_table_config_new(etable_view->title,
 				    etable_view->spec,
-				    etable_view->state);
+				    etable_view->state,
+				    parent);
 
-	gtk_signal_connect(GTK_OBJECT(config), "changed",
-			   GTK_SIGNAL_FUNC(config_changed), view);
+	g_signal_connect(config, "changed",
+			 G_CALLBACK(config_changed), view);
 }
 
 static void  
@@ -79,8 +106,8 @@ gal_view_etable_get_title       (GalView *view)
 }
 
 static void
-gal_view_etable_set_title       (GalView *view,
-				 const char *title)
+gal_view_etable_set_title (GalView *view,
+			   const char *title)
 {
 	g_free(GAL_VIEW_ETABLE(view)->title);
 	GAL_VIEW_ETABLE(view)->title = g_strdup(title);
@@ -99,35 +126,43 @@ gal_view_etable_clone       (GalView *view)
 
 	gve = GAL_VIEW_ETABLE(view);
 
-	new        = gtk_type_new (gal_view_etable_get_type ());
+	new        = g_object_new (GAL_VIEW_ETABLE_TYPE, NULL);
 	new->spec  = gve->spec;
 	new->title = g_strdup (gve->title);
 	new->state = e_table_state_duplicate(gve->state);
 
-	gtk_object_ref(GTK_OBJECT(new->spec));
+	g_object_ref(new->spec);
 
 	return GAL_VIEW(new);
 }
 
 static void
-gal_view_etable_destroy         (GtkObject *object)
+gal_view_etable_dispose         (GObject *object)
 {
 	GalViewEtable *view = GAL_VIEW_ETABLE(object);
-	g_free(view->title);
-	if (view->spec)
-		gtk_object_unref(GTK_OBJECT(view->spec));
-	if (view->state)
-		gtk_object_unref(GTK_OBJECT(view->state));
 
-	if (GTK_OBJECT_CLASS (gal_view_etable_parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (gal_view_etable_parent_class)->destroy) (object);
+	gal_view_etable_detach (view);
+
+	g_free(view->title);
+	view->title = NULL;
+
+	if (view->spec)
+		g_object_unref(view->spec);
+	view->spec = NULL;
+
+	if (view->state)
+		g_object_unref(view->state);
+	view->state = NULL;
+
+	if (G_OBJECT_CLASS (gal_view_etable_parent_class)->dispose)
+		(* G_OBJECT_CLASS (gal_view_etable_parent_class)->dispose) (object);
 }
 
 static void
-gal_view_etable_class_init      (GtkObjectClass *object_class)
+gal_view_etable_class_init      (GObjectClass *object_class)
 {
 	GalViewClass *gal_view_class  = GAL_VIEW_CLASS(object_class);
-	gal_view_etable_parent_class  = gtk_type_class (PARENT_TYPE);
+	gal_view_etable_parent_class  = g_type_class_ref (PARENT_TYPE);
 
 	gal_view_class->edit          = gal_view_etable_edit         ;
 	gal_view_class->load          = gal_view_etable_load         ;
@@ -137,7 +172,7 @@ gal_view_etable_class_init      (GtkObjectClass *object_class)
 	gal_view_class->get_type_code = gal_view_etable_get_type_code;
 	gal_view_class->clone         = gal_view_etable_clone        ;
 
-	object_class->destroy         = gal_view_etable_destroy      ;
+	object_class->dispose         = gal_view_etable_dispose      ;
 }
 
 static void
@@ -148,6 +183,8 @@ gal_view_etable_init      (GalViewEtable *gve)
 	gve->title = NULL;
 }
 
+E_MAKE_TYPE(gal_view_etable, "GalViewEtable", GalViewEtable, gal_view_etable_class_init, gal_view_etable_init, PARENT_TYPE)
+
 /**
  * gal_view_etable_new
  * @spec: The ETableSpecification that this view will be based upon.
@@ -162,7 +199,7 @@ GalView *
 gal_view_etable_new (ETableSpecification *spec,
 		     const gchar *title)
 {
-	return gal_view_etable_construct (gtk_type_new (gal_view_etable_get_type ()), spec, title);
+	return gal_view_etable_construct (g_object_new (GAL_VIEW_ETABLE_TYPE, NULL), spec, title);
 }
 
 /**
@@ -182,11 +219,11 @@ gal_view_etable_construct  (GalViewEtable *view,
 			    const gchar *title)
 {
 	if (spec)
-		gtk_object_ref(GTK_OBJECT(spec));
+		g_object_ref(spec);
 	view->spec = spec;
 
 	if (view->state)
-		gtk_object_unref(GTK_OBJECT(view->state));
+		g_object_unref(view->state);
 	view->state = e_table_state_duplicate(spec->state);
 
 	view->title = g_strdup(title);
@@ -194,27 +231,73 @@ gal_view_etable_construct  (GalViewEtable *view,
 	return GAL_VIEW(view);
 }
 
-GtkType
-gal_view_etable_get_type        (void)
+void
+gal_view_etable_set_state (GalViewEtable *view, ETableState *state)
 {
-	static guint type = 0;
-	
-	if (!type)
-	{
-		GtkTypeInfo info =
-		{
-			"GalViewEtable",
-			sizeof (GalViewEtable),
-			sizeof (GalViewEtableClass),
-			(GtkClassInitFunc) gal_view_etable_class_init,
-			(GtkObjectInitFunc) gal_view_etable_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-		
-		type = gtk_type_unique (PARENT_TYPE, &info);
-	}
+	if (view->state)
+		g_object_unref(view->state);
+	view->state = e_table_state_duplicate(state);
 
-	return type;
+	gal_view_changed(GAL_VIEW(view));
+}
+
+static void
+table_state_changed (ETable *table, GalViewEtable *view)
+{
+	ETableState *state;
+
+	state = e_table_get_state_object (table);
+	g_object_unref (view->state);
+	view->state = state;
+
+	gal_view_changed(GAL_VIEW(view));
+}
+
+static void
+tree_state_changed (ETree *tree, GalViewEtable *view)
+{
+	ETableState *state;
+
+	state = e_tree_get_state_object (tree);
+	g_object_unref (view->state);
+	view->state = state;
+
+	gal_view_changed(GAL_VIEW(view));
+}
+
+void
+gal_view_etable_attach_table (GalViewEtable *view, ETable *table)
+{
+	gal_view_etable_detach (view);
+
+	view->table = table;
+
+	e_table_set_state_object(view->table, view->state);
+	g_object_ref (view->table);
+	view->table_state_changed_id =
+		g_signal_connect(view->table, "state_change",
+				 G_CALLBACK (table_state_changed), view);
+}
+
+void
+gal_view_etable_attach_tree (GalViewEtable *view, ETree *tree)
+{
+	gal_view_etable_detach (view);
+
+	view->tree = tree;
+
+	e_tree_set_state_object(view->tree, view->state);
+	g_object_ref (view->tree);
+	view->tree_state_changed_id =
+		g_signal_connect(view->tree, "state_change",
+				 G_CALLBACK (tree_state_changed), view);
+}
+
+void
+gal_view_etable_detach (GalViewEtable *view)
+{
+	if (view->table != NULL)
+		detach_table (view);
+	if (view->tree != NULL)
+		detach_tree (view);
 }
