@@ -94,13 +94,13 @@ gal_view_new_dialog_init (GalViewNewDialog *dialog)
 		return;
 	}
 	gtk_widget_ref(widget);
-	gtk_widget_unparent(widget);
+	gtk_container_remove (GTK_CONTAINER (widget->parent), widget);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), widget, TRUE, TRUE, 0);
 	gtk_widget_unref(widget);
 
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-				GTK_STOCK_OK, GTK_RESPONSE_OK,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_OK, GTK_RESPONSE_OK,
 				NULL);
 
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, TRUE, FALSE);
