@@ -199,7 +199,6 @@ e_group_bar_dispose (GObject *object)
 	group_bar->children = NULL;
 }
 
-
 static void
 e_group_bar_realize (GtkWidget *widget)
 {
@@ -237,7 +236,7 @@ e_group_bar_realize (GtkWidget *widget)
 
 	widget->style = gtk_style_attach (widget->style, widget->window);
 	gtk_style_set_background (widget->style, widget->window,
-				  GTK_STATE_NORMAL);
+				  GTK_STATE_ACTIVE);
 
 	gdk_window_set_back_pixmap (widget->window, NULL, FALSE);
 
