@@ -310,7 +310,7 @@ static void
 e_icon_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
 	EIconBar *icon_bar;
-	gint canvas_width, canvas_height, height;
+	gint canvas_width, canvas_height, height, val;
 
 	icon_bar = E_ICON_BAR (widget);
 
@@ -323,11 +323,13 @@ e_icon_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	   the canvas, and reset the button labels, so they fit. */
 	e_icon_bar_recalc_common_positions (icon_bar);
 	height = e_icon_bar_recalc_item_positions (icon_bar);
+	val = GTK_LAYOUT (widget)->vadjustment->value;
 
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (widget),
 					0, 0, canvas_width,
 					MAX (height, canvas_height - 1));
 
+	GTK_LAYOUT (widget)->vadjustment->value = val;
 	/* If we are editing an item, make sure it is visible. */
 	e_icon_bar_ensure_edited_item_visible (icon_bar);
 
