@@ -63,6 +63,8 @@ static void e_shortcut_bar_init			(EShortcutBar	*shortcut_bar);
 static void e_shortcut_bar_dispose		(GObject	*object);
 static void e_shortcut_bar_finalize		(GObject	*object);
 
+static void e_shortcut_bar_style_set (GtkWidget *widget, GtkStyle *prev_style);
+
 static void e_shortcut_bar_disconnect_model	(EShortcutBar	*shortcut_bar,
 						 gboolean        model_destroyed);
 
@@ -255,6 +257,8 @@ e_shortcut_bar_class_init (EShortcutBarClass *class)
 	/* Method override */
 	object_class->dispose  = e_shortcut_bar_dispose;
 	object_class->finalize = e_shortcut_bar_finalize;
+
+	widget_class->style_set = e_shortcut_bar_style_set;
 }
 
 
@@ -314,6 +318,26 @@ e_shortcut_bar_finalize (GObject *object)
 	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+static void
+e_shortcut_bar_style_set (GtkWidget *widget, GtkStyle *prev_style)
+{
+	EShortcutBar *shortcut_bar;
+	EShortcutBarGroup *group;
+	gint num_groups, group_num;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (E_IS_SHORTCUT_BAR (widget));
+
+	shortcut_bar = E_SHORTCUT_BAR (widget);
+
+	num_groups = shortcut_bar->groups->len;
+	for (group_num = 0; group_num < num_groups; group_num++) {
+		group = &g_array_index (shortcut_bar->groups,
+					EShortcutBarGroup, group_num);
+		e_shortcut_bar_set_canvas_style (shortcut_bar,
+						 group->icon_bar);
+	}
+}
 
 void
 e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
@@ -462,7 +486,7 @@ e_shortcut_bar_on_item_updated (EShortcutModel *model,
 				    item_url, item_name, image);
 }
 
-static void
+static gboolean
 e_shortcut_bar_group_button_press (GtkWidget *widget,
 				   GdkEventButton *button_event,
 				   EShortcutBar *shortcut_bar)
@@ -480,9 +504,12 @@ e_shortcut_bar_group_button_press (GtkWidget *widget,
 			g_signal_emit (shortcut_bar,
 				       e_shortcut_bar_signals[ITEM_SELECTED], 0,
 				       button_event, i, -1);
+
 			break;
 		}
 	}
+
+	return FALSE;
 }
 
 static gint
@@ -554,7 +581,7 @@ e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
 
 	label = e_entry_new ();
 	g_object_set(label,
-		     "draw_background", FALSE,
+		     "draw_background", TRUE,
 		     "draw_borders", FALSE,
 		     "draw_button", TRUE,
 		     "editable", FALSE,
@@ -670,21 +697,11 @@ static void
 e_shortcut_bar_set_canvas_style (EShortcutBar *shortcut_bar,
 				 GtkWidget *canvas)
 {
-        GtkRcStyle *rc_style;
+	GtkStyle *style;
 
-        rc_style = gtk_rc_style_new ();
+	style = gtk_rc_get_style (GTK_WIDGET (shortcut_bar));
 
-        rc_style->color_flags[GTK_STATE_NORMAL] = GTK_RC_FG | GTK_RC_BG;
-        rc_style->fg[GTK_STATE_NORMAL].red   = 65535;
-        rc_style->fg[GTK_STATE_NORMAL].green = 65535;
-        rc_style->fg[GTK_STATE_NORMAL].blue  = 65535;
-
-        rc_style->bg[GTK_STATE_NORMAL].red   = 0x8000;
-        rc_style->bg[GTK_STATE_NORMAL].green = 0x8000;
-        rc_style->bg[GTK_STATE_NORMAL].blue  = 0x8000;
-
-        gtk_widget_modify_style (GTK_WIDGET (canvas), rc_style);
-        gtk_rc_style_unref (rc_style);
+        gtk_widget_modify_bg (GTK_WIDGET (canvas), GTK_STATE_NORMAL, &style->bg[GTK_STATE_ACTIVE]);
 }
 
 
