@@ -30,6 +30,7 @@
 #include <gtk/gtkarrow.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkeventbox.h>
 #include "e-vscrolled-bar.h"
 #include <gal/util/e-util.h>
 
@@ -51,6 +52,8 @@ static void e_vscrolled_bar_size_request (GtkWidget      *widget,
 					  GtkRequisition *requisition);
 static void e_vscrolled_bar_size_allocate (GtkWidget *widget,
 					   GtkAllocation *allocation);
+static int  e_vscrolled_bar_expose (GtkWidget      *widget,
+				    GdkEventExpose *event);
 static gint e_vscrolled_bar_button_press   (GtkWidget        *widget,
 					    GdkEventButton   *event);
 static void e_vscrolled_bar_add (GtkContainer *container,
@@ -105,6 +108,7 @@ e_vscrolled_bar_class_init (EVScrolledBarClass *class)
 	widget_class->size_request	= e_vscrolled_bar_size_request;
  	widget_class->size_allocate	= e_vscrolled_bar_size_allocate;
 	widget_class->button_press_event= e_vscrolled_bar_button_press;
+	widget_class->expose_event      = e_vscrolled_bar_expose;
 
 	container_class->add		= e_vscrolled_bar_add;
 	container_class->remove		= e_vscrolled_bar_remove;
@@ -122,18 +126,22 @@ e_vscrolled_bar_init (EVScrolledBar *vscrolled_bar)
 	gtk_container_set_resize_mode (GTK_CONTAINER (vscrolled_bar),
 				       GTK_RESIZE_QUEUE);
 
+	gtk_widget_set_redraw_on_allocate (GTK_WIDGET (vscrolled_bar), TRUE);
+
 	gtk_widget_push_composite_child ();
 
+	vscrolled_bar->up_ebox = gtk_event_box_new ();
 	vscrolled_bar->up_button = gtk_button_new ();
 	gtk_widget_set_composite_name (vscrolled_bar->up_button,
 				       "up_button");
-	gtk_widget_set_parent (vscrolled_bar->up_button,
-			       GTK_WIDGET (vscrolled_bar));
+	gtk_widget_set_parent (vscrolled_bar->up_ebox, GTK_WIDGET (vscrolled_bar));
 	arrow = gtk_arrow_new (GTK_ARROW_UP, GTK_SHADOW_OUT);
 	gtk_misc_set_padding (GTK_MISC (arrow), 1, 1);
 	gtk_widget_show (arrow);
 	gtk_container_add (GTK_CONTAINER (vscrolled_bar->up_button), arrow);
 	gtk_widget_show (vscrolled_bar->up_button);
+	gtk_container_add (GTK_CONTAINER (vscrolled_bar->up_ebox), vscrolled_bar->up_button);
+	gtk_widget_show (vscrolled_bar->up_ebox);
 	g_signal_connect_after (vscrolled_bar->up_button, "pressed",
 				G_CALLBACK (e_vscrolled_bar_button_pressed), vscrolled_bar);
 	g_signal_connect_after (vscrolled_bar->up_button, "released",
@@ -141,16 +149,19 @@ e_vscrolled_bar_init (EVScrolledBar *vscrolled_bar)
 	g_signal_connect (vscrolled_bar->up_button, "clicked",
 			  G_CALLBACK (e_vscrolled_bar_button_clicked), vscrolled_bar);
 
+	vscrolled_bar->down_ebox = gtk_event_box_new ();
 	vscrolled_bar->down_button = gtk_button_new ();
 	gtk_widget_set_composite_name (vscrolled_bar->up_button,
 				       "down_button");
-	gtk_widget_set_parent (vscrolled_bar->down_button,
+	gtk_widget_set_parent (vscrolled_bar->down_ebox,
 			       GTK_WIDGET (vscrolled_bar));
 	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
 	gtk_misc_set_padding (GTK_MISC (arrow), 1, 1);
 	gtk_widget_show (arrow);
 	gtk_container_add (GTK_CONTAINER (vscrolled_bar->down_button), arrow);
 	gtk_widget_show (vscrolled_bar->down_button);
+	gtk_container_add (GTK_CONTAINER (vscrolled_bar->down_ebox), vscrolled_bar->down_button);
+	gtk_widget_show (vscrolled_bar->down_ebox);
 	g_signal_connect_after (vscrolled_bar->down_button, "pressed",
 				G_CALLBACK (e_vscrolled_bar_button_pressed), vscrolled_bar);
 	g_signal_connect_after (vscrolled_bar->down_button, "released",
@@ -201,13 +212,13 @@ e_vscrolled_bar_dispose (GObject *object)
 		vscrolled_bar->timeout_id = 0;
 	}
 
-	if (vscrolled_bar->up_button)
-		gtk_widget_unparent (vscrolled_bar->up_button);
-	vscrolled_bar->up_button = NULL;
+	if (vscrolled_bar->up_ebox)
+		gtk_widget_unparent (vscrolled_bar->up_ebox);
+	vscrolled_bar->up_ebox = NULL;
 
-	if (vscrolled_bar->down_button)
-		gtk_widget_unparent (vscrolled_bar->down_button);
-	vscrolled_bar->down_button = NULL;
+	if (vscrolled_bar->down_ebox)
+		gtk_widget_unparent (vscrolled_bar->down_ebox);
+	vscrolled_bar->down_ebox = NULL;
 
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -242,15 +253,15 @@ e_vscrolled_bar_map (GtkWidget *widget)
   
 	adjustment = vscrolled_bar->adjustment;
 
-	if (GTK_WIDGET_VISIBLE (vscrolled_bar->up_button)
+	if (GTK_WIDGET_VISIBLE (vscrolled_bar->up_ebox)
 	    && adjustment->value != adjustment->lower
-	    && !GTK_WIDGET_MAPPED (vscrolled_bar->up_button))
-		gtk_widget_map (vscrolled_bar->up_button);
-  
-	if (GTK_WIDGET_VISIBLE (vscrolled_bar->down_button)
+	    && !GTK_WIDGET_MAPPED (vscrolled_bar->up_ebox))
+		gtk_widget_map (vscrolled_bar->up_ebox);
+	
+	if (GTK_WIDGET_VISIBLE (vscrolled_bar->down_ebox)
 	    && adjustment->value < adjustment->upper - adjustment->page_size
-	    && !GTK_WIDGET_MAPPED (vscrolled_bar->down_button))
-		gtk_widget_map (vscrolled_bar->down_button);
+	    && !GTK_WIDGET_MAPPED (vscrolled_bar->down_ebox))
+		gtk_widget_map (vscrolled_bar->down_ebox);
 }
 
 
@@ -267,11 +278,11 @@ e_vscrolled_bar_unmap (GtkWidget *widget)
 	/* chain parent class handler to unmap self and child */
 	GTK_WIDGET_CLASS (parent_class)->unmap (widget);
   
-	if (GTK_WIDGET_MAPPED (vscrolled_bar->up_button))
-		gtk_widget_unmap (vscrolled_bar->up_button);
+	if (GTK_WIDGET_MAPPED (vscrolled_bar->up_ebox))
+		gtk_widget_unmap (vscrolled_bar->up_ebox);
   
-	if (GTK_WIDGET_MAPPED (vscrolled_bar->down_button))
-		gtk_widget_unmap (vscrolled_bar->down_button);
+	if (GTK_WIDGET_MAPPED (vscrolled_bar->down_ebox))
+		gtk_widget_unmap (vscrolled_bar->down_ebox);
 }
 
 
@@ -301,12 +312,12 @@ e_vscrolled_bar_size_request (GtkWidget      *widget,
 	}
 
 	/* We remember the requested heights of the up & down buttons. */
-	gtk_widget_size_request (vscrolled_bar->up_button,
+	gtk_widget_size_request (vscrolled_bar->up_ebox,
 				 &child_requisition);
 	vscrolled_bar->up_button_width = child_requisition.width;
 	vscrolled_bar->up_button_height = child_requisition.height;
 
-	gtk_widget_size_request (vscrolled_bar->down_button,
+	gtk_widget_size_request (vscrolled_bar->down_ebox,
 				 &child_requisition);
 	vscrolled_bar->down_button_width = child_requisition.width;
 	vscrolled_bar->down_button_height = child_requisition.height;
@@ -337,62 +348,51 @@ e_vscrolled_bar_size_allocate (GtkWidget *widget,
 
 	border_width = GTK_CONTAINER (widget)->border_width;
 
-	child_allocation.x = border_width;
-	child_allocation.y = border_width;
+	child_allocation.x = allocation->x + border_width;
+	child_allocation.y = allocation->y + border_width;
 	child_allocation.width = allocation->width - 2 * border_width;
 	child_allocation.height = allocation->height - 2 * border_width;
 	gtk_widget_size_allocate (bin->child, &child_allocation);
 
-	button_allocation.x = allocation->width - border_width
+	button_allocation.x = allocation->x + allocation->width - border_width
 		- vscrolled_bar->up_button_width
 		- E_VSCROLLED_BAR_BUTTON_X_OFFSET;
-	button_allocation.y = border_width + E_VSCROLLED_BAR_BUTTON_Y_OFFSET;
+	button_allocation.y = allocation->y + border_width + E_VSCROLLED_BAR_BUTTON_Y_OFFSET;
 	button_allocation.width = vscrolled_bar->up_button_width;
 	button_allocation.height = vscrolled_bar->up_button_height;
-	gtk_widget_size_allocate (vscrolled_bar->up_button,
+	gtk_widget_size_allocate (vscrolled_bar->up_ebox,
 				  &button_allocation);
 
-	button_allocation.x = allocation->width - border_width
+	button_allocation.x = allocation->x + allocation->width - border_width
 		- vscrolled_bar->down_button_width
 		- E_VSCROLLED_BAR_BUTTON_X_OFFSET;
-	button_allocation.y = allocation->height - border_width
+	button_allocation.y = allocation->y + allocation->height - border_width
 		- vscrolled_bar->down_button_height
 		- E_VSCROLLED_BAR_BUTTON_Y_OFFSET;
 	button_allocation.width = vscrolled_bar->down_button_width;
 	button_allocation.height = vscrolled_bar->down_button_height;
-	gtk_widget_size_allocate (vscrolled_bar->down_button,
+	gtk_widget_size_allocate (vscrolled_bar->down_ebox,
 				  &button_allocation);
 }
 
-#if 0
-static void
-e_vscrolled_bar_draw (GtkWidget    *widget,
-		      GdkRectangle *area)
+static int
+e_vscrolled_bar_expose (GtkWidget      *widget,
+			GdkEventExpose *event)
 {
-	EVScrolledBar *vscrolled_bar;
-	GtkBin *bin;
-	GdkRectangle child_area;
+	EVScrolledBar *vscrolled_bar = E_VSCROLLED_BAR (widget);
+	GtkContainer *container = GTK_CONTAINER (vscrolled_bar);
+	GtkAdjustment *adjustment = vscrolled_bar->adjustment;
 
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (E_IS_VSCROLLED_BAR (widget));
-	g_return_if_fail (area != NULL);
-  
-	vscrolled_bar = E_VSCROLLED_BAR (widget);
-	bin = GTK_BIN (widget);
+	if (GTK_BIN (vscrolled_bar)->child != NULL)
+		gtk_container_propagate_expose (container, GTK_BIN (vscrolled_bar)->child, event);
 
-	if (bin->child && GTK_WIDGET_VISIBLE (bin->child) &&
-	    gtk_widget_intersect (bin->child, area, &child_area))
-		gtk_widget_draw (bin->child, &child_area);
-  
-	if (GTK_WIDGET_DRAWABLE (vscrolled_bar->up_button) &&
-	    gtk_widget_intersect (vscrolled_bar->up_button, area, &child_area))
-		gtk_widget_draw (vscrolled_bar->up_button, &child_area);
-  
-	if (GTK_WIDGET_DRAWABLE (vscrolled_bar->down_button) &&
-	    gtk_widget_intersect (vscrolled_bar->down_button, area, &child_area))
-		gtk_widget_draw (vscrolled_bar->down_button, &child_area);
+	if (adjustment->value != adjustment->lower)
+		gtk_container_propagate_expose (container, vscrolled_bar->up_ebox, event);
+	if (adjustment->value < adjustment->upper - adjustment->page_size)
+		gtk_container_propagate_expose (container, vscrolled_bar->down_ebox, event);
+
+	return FALSE;
 }
-#endif
 
 static gint
 e_vscrolled_bar_button_press (GtkWidget      *widget,
@@ -494,16 +494,6 @@ e_vscrolled_bar_forall (GtkContainer *container,
 						    include_internals,
 						    callback,
 						    callback_data);
-	if (include_internals) {
-		EVScrolledBar *vscrolled_bar;
-
-		vscrolled_bar = E_VSCROLLED_BAR (container);
-      
-		if (vscrolled_bar->up_button)
-			callback (vscrolled_bar->up_button, callback_data);
-		if (vscrolled_bar->down_button)
-			callback (vscrolled_bar->down_button, callback_data);
-	}
 }
 
 
@@ -595,19 +585,19 @@ e_vscrolled_bar_adjustment_changed (GtkAdjustment *adjustment,
 
 	/* If the adjustment value is not 0, show the up button. */
 	if (adjustment->value != adjustment->lower) {
-		gtk_widget_map (vscrolled_bar->up_button);
-		gdk_window_raise (vscrolled_bar->up_button->window);
+		gtk_widget_map (vscrolled_bar->up_ebox);
+		gdk_window_raise (vscrolled_bar->up_ebox->window);
 	} else {
-		gtk_widget_unmap (vscrolled_bar->up_button);
+		gtk_widget_unmap (vscrolled_bar->up_ebox);
 	}
 
 	/* If the adjustment value is less than the maximum value, show the
 	   down button. */
 	if (adjustment->value < adjustment->upper - adjustment->page_size) {
-		gtk_widget_map (vscrolled_bar->down_button);
-		gdk_window_raise (vscrolled_bar->down_button->window);
+		gtk_widget_map (vscrolled_bar->down_ebox);
+		gdk_window_raise (vscrolled_bar->down_ebox->window);
 	} else {
-		gtk_widget_unmap (vscrolled_bar->down_button);
+		gtk_widget_unmap (vscrolled_bar->down_ebox);
 	}
 }
 
