@@ -1,20 +1,32 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * e-selection-model-array.c: a Selection Model
+ * e-bit-array.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Christopher James Lahey <clahey@ximian.com>
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 2000, 2001 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <gtk/gtksignal.h>
 #include "e-bit-array.h"
 #include "gal/util/e-util.h"
 
-#define EBA_CLASS(e) ((EBitArrayClass *)((GtkObject *)e)->klass)
-
-#define PARENT_TYPE (gtk_object_get_type ())
+#define PARENT_TYPE G_TYPE_OBJECT
 
 #define ONES ((guint32) 0xffffffff)
 
@@ -24,7 +36,7 @@
 #define BITMASK_LEFT(n) ((((n) % 32) == 0) ? 0 : (ONES << (32 - ((n) % 32))))
 #define BITMASK_RIGHT(n) ((guint32)(((guint32) ONES) >> ((n) % 32)))
 
-static GtkObjectClass *parent_class;
+static GObjectClass *parent_class;
 
 static void
 e_bit_array_insert_real(EBitArray *eba, int row)
@@ -34,7 +46,7 @@ e_bit_array_insert_real(EBitArray *eba, int row)
 	if(eba->bit_count >= 0) {
 		/* Add another word if needed. */
 		if ((eba->bit_count & 0x1f) == 0) {
-			eba->data = g_renew(gint, eba->data, (eba->bit_count >> 5) + 1);
+			eba->data = g_renew(guint32, eba->data, (eba->bit_count >> 5) + 1);
 			eba->data[eba->bit_count >> 5] = 0;
 		}
 
@@ -83,7 +95,7 @@ e_bit_array_delete_real(EBitArray *eba, int row, gboolean move_selection_mode)
 		eba->bit_count --;
 		/* Remove the last word if not needed. */
 		if ((eba->bit_count & 0x1f) == 0) {
-			eba->data = g_renew(gint, eba->data, eba->bit_count >> 5);
+			eba->data = g_renew(guint32, eba->data, eba->bit_count >> 5);
 		}
 		if (move_selection_mode && selected) {
 			e_bit_array_select_single_row (eba, row > 0 ? row - 1 : 0);
@@ -127,16 +139,18 @@ e_bit_array_move_row(EBitArray *eba, int old_row, int new_row)
 }
 
 static void
-eba_destroy (GtkObject *object)
+eba_dispose (GObject *object)
 {
 	EBitArray *eba;
 
 	eba = E_BIT_ARRAY (object);
 
-	g_free(eba->data);
+	if (eba->data)
+		g_free(eba->data);
+	eba->data = NULL;
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
 /** 
@@ -252,7 +266,7 @@ e_bit_array_select_all (EBitArray *eba)
 	int i;
 	
 	if (!eba->data)
-		eba->data = g_new0 (gint, (eba->bit_count + 31) / 32);
+		eba->data = g_new0 (guint32, (eba->bit_count + 31) / 32);
 	
 	for (i = 0; i < (eba->bit_count + 31) / 32; i ++) {
 		eba->data[i] = ONES;
@@ -284,7 +298,7 @@ e_bit_array_invert_selection (EBitArray *eba)
 	int i;
 
 	if (!eba->data)
-		eba->data = g_new0 (gint, (eba->bit_count + 31) / 32);
+		eba->data = g_new0 (guint32, (eba->bit_count + 31) / 32);
 	
 	for (i = 0; i < (eba->bit_count + 31) / 32; i ++) {
 		eba->data[i] = ~eba->data[i];
@@ -365,7 +379,7 @@ e_bit_array_select_single_row (EBitArray *eba, int row)
 		if (!((i == BOX(row) && eba->data[i] == BITMASK(row)) ||
 		      (i != BOX(row) && eba->data[i] == 0))) {
 			g_free(eba->data);
-			eba->data = g_new0(gint, (eba->bit_count + 31) / 32);
+			eba->data = g_new0(guint32, (eba->bit_count + 31) / 32);
 			eba->data[BOX(row)] = BITMASK(row);
 
 			break;
@@ -393,23 +407,23 @@ e_bit_array_init (EBitArray *eba)
 static void
 e_bit_array_class_init (EBitArrayClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
-	object_class = GTK_OBJECT_CLASS(klass);
+	object_class = G_OBJECT_CLASS(klass);
 
-	object_class->destroy = eba_destroy;
+	object_class->dispose = eba_dispose;
 }
 
 E_MAKE_TYPE(e_bit_array, "EBitArray", EBitArray,
-	    e_bit_array_class_init, e_bit_array_init, PARENT_TYPE);
+	    e_bit_array_class_init, e_bit_array_init, PARENT_TYPE)
 
 EBitArray *
 e_bit_array_new (int count)
 {
-	EBitArray *eba = gtk_type_new (e_bit_array_get_type ());
+	EBitArray *eba = g_object_new (E_BIT_ARRAY_TYPE, NULL);
 	eba->bit_count = count;
-	eba->data = g_new0(gint, (eba->bit_count + 31) / 32);
+	eba->data = g_new0(guint32, (eba->bit_count + 31) / 32);
 	return eba;
 }
