@@ -198,7 +198,7 @@ e_write_file(const char *filename, const char *data, int flags)
 	int fd;
 	int length = strlen(data);
 	int bytes;
-	fd = open(filename, flags, 0666);
+	fd = open(filename, flags | O_WRONLY, 0666);
 	if (fd == -1)
 		return errno;
 	while (length > 0) {
@@ -275,11 +275,8 @@ e_mkdir_hier(const char *path, mode_t mode)
 		p = strchr (p + 1, '/');
 		if (p)
 			*p = '\0';
-		if (mkdir (copy, mode) == -1) {
-			switch (errno) {
-			case EEXIST:
-				break;
-			default:
+		if (access (copy, F_OK) == -1) {
+			if (mkdir (copy, mode) == -1) {
 				g_free (copy);
 				return -1;
 			}
