@@ -31,17 +31,19 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define E_CATEGORIES_MASTER_LIST_TYPE        (e_categories_master_list_get_type ())
-#define E_CATEGORIES_MASTER_LIST(o)          (GTK_CHECK_CAST ((o), E_CATEGORIES_MASTER_LIST_TYPE, ECategoriesMasterList))
-#define E_CATEGORIES_MASTER_LIST_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_CATEGORIES_MASTER_LIST_TYPE, ECategoriesMasterListClass))
-#define E_IS_CATEGORIES_MASTER_LIST(o)       (GTK_CHECK_TYPE ((o), E_CATEGORIES_MASTER_LIST_TYPE))
-#define E_IS_CATEGORIES_MASTER_LIST_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), E_CATEGORIES_MASTER_LIST_TYPE))
+#define E_CATEGORIES_MASTER_LIST(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), E_CATEGORIES_MASTER_LIST_TYPE, ECategoriesMasterList))
+#define E_CATEGORIES_MASTER_LIST_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST((k), E_CATEGORIES_MASTER_LIST_TYPE, ECategoriesMasterListClass))
+#define E_IS_CATEGORIES_MASTER_LIST(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), E_CATEGORIES_MASTER_LIST_TYPE))
+#define E_IS_CATEGORIES_MASTER_LIST_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), E_CATEGORIES_MASTER_LIST_TYPE))
+#define E_CATEGORIES_MASTER_LIST_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), E_CATEGORIES_MASTER_LIST_TYPE, ECategoriesMasterListClass))
+
 
 typedef struct {
-	GtkObject   base;
+	GObject   base;
 } ECategoriesMasterList;
 
 typedef struct {
-	GtkObjectClass parent_class;
+	GObjectClass parent_class;
 
 	/*
 	 * Virtual methods
@@ -61,7 +63,7 @@ typedef struct {
 	void        (*changed) (ECategoriesMasterList *etm);
 } ECategoriesMasterListClass;
 
-GtkType     e_categories_master_list_get_type  (void);
+GType       e_categories_master_list_get_type  (void);
 
 /**/
 int         e_categories_master_list_count     (ECategoriesMasterList *ecml);
