@@ -126,8 +126,8 @@ e_categories_class_init (ECategoriesClass *klass)
 
 	g_object_class_install_property (object_class, PROP_ECML,
 					 g_param_spec_object ("ecml",
-							      _( "ECML" ),
-							      _( "ECategoriesMasterListCombo" ),
+							      "ECML",
+							      "ECategoriesMasterListCombo",
 							      E_CATEGORIES_MASTER_LIST_TYPE,
 							      G_PARAM_READWRITE));
 }
