@@ -61,7 +61,7 @@ class_init (EHScrollbarClass *klass)
 {
 	GtkWidgetClass *widget_class;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
 	widget_class = GTK_WIDGET_CLASS (klass);
 
@@ -80,7 +80,7 @@ e_hscrollbar_new (GtkAdjustment *adjustment)
 {
 	EHScrollbar *new;
 
-	new = E_HSCROLLBAR (gtk_object_new (e_hscrollbar_get_type (), "adjustment", adjustment, NULL));
+	new = E_HSCROLLBAR (g_object_new (E_TYPE_HSCROLLBAR, "adjustment", adjustment, NULL));
 	
 	return GTK_WIDGET (new);
 }
